#ifndef _IOUTIL_H
#define	_IOUTIL_H

#ident "@(#)$Header: ioutil.h,v 1.5 94/04/13 16:59:05 wob Exp $ SNI"

#include <limits.h>

/*
 * Name conversion utilities
 */
struct name_parts {
	enum { OTHER, DISK, TAPE, VDISK } type;
	char	rooted	[sizeof("/")];
	char	dev	[sizeof("dev/")];
	char	subdir	[sizeof("iop0/") + 32];
	char	raw	[sizeof("r")];
	char	typestr	[sizeof("disk") + 32];
	char	units	[sizeof("000") + 32];
	char	options	[sizeof("s15") + 32];
};

extern struct name_parts nameparts(const char *name);
#if !defined (__STRICT_ANSI__) || defined(_POSIX_SOURCE) || defined(_XOPEN_SOURCE)
extern char *	rawdevname(const char *device, char path[PATH_MAX]);
extern char *	blkdevname(const char *device, char path[PATH_MAX]);
extern char *	autoconfname(const char *device, char path[PATH_MAX]);
#else
extern char *   rawdevname(const char *device, char path[__PATH_MAX]);
extern char *   blkdevname(const char *device, char path[__PATH_MAX]);
extern char *   autoconfname(const char *device, char path[__PATH_MAX]);
#endif

/*
 * Device management interfaces
 */

/* autoconf tree access */
struct ac_node {
	struct ac_node	*ac_parent;
	struct ac_node	*ac_child;
	struct ac_node	*ac_next;
	void		*ac_kaddr;
	struct autoconf	*ac;
};

const struct ac_node *ac_readtree(void);
void ac_freetree(const struct ac_node *tree);
const struct ac_node *ac_findnode(const char *name, const struct ac_node *tree);
int ac_state(const struct ac_node *node, long *state);
void ac_traverse(const struct ac_node *tree,
		void (*func)(const struct ac_node *node, int depth, void *arg),
		void *arg);

/* device repair/replacement primitives */
enum io_repair_cmd {
	IOR_GODOWN,
	IOR_BACKUP,
	IOR_GAVEUP,
	IOR_GRAB,
	IOR_LETGO,
	IOR_POWERDOWN,
	IOR_PROBE,
	IOR_DECONFIG,
	IOR_LOCK,
	IOR_UNLOCK,
	IOR_ISDOWN,
	IOR_FLUSHQ,
	IOR_ENOPEN,
	IOR_DISOPEN,
	IOR_DEVRST,
	IOR_BUSRST
};
static char *io_repair_name []={
	"IOR_GODOWN",
	"IOR_BACKUP",
	"IOR_GAVEUP",
	"IOR_GRAB",
	"IOR_LETGO",
	"IOR_POWERDOWN",
	"IOR_PROBE",
	"IOR_DECONFIG",
	"IOR_LOCK",
	"IOR_UNLOCK",
	"IOR_ISDOWN",
	"IOR_FLUSHQ",
	"IOR_ENOPEN",
	"IOR_DISOPEN",
	"IOR_DEVRST",
	"IOR_BUSRST"
};
int io_repair(enum io_repair_cmd command, const struct ac_node *subtree);

#endif	/* _IOUTIL_H */
