#ident "@(#)$Header: cpupriv.h,v 1.13.1.1 94/04/28 14:21:49 hn Exp $ SNI"
/*
 * "@(#)cpupriv.h	Revision: 3.4	93/01/19 00:45:32"
 */

/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990, 1991 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _SYS_CPUPRIV_H
#define _SYS_CPUPRIV_H

/* A number of critical data structures are cloned one per CPU to avoid
 * using spinlocks. This is good since generally these structures contain
 * things that are changed a lot (page fault counts, context information).
 *
 * To make life even simpler, we now put these structures together on a
 * single PAGESIZE page (4K for R2/3000 processors) per CPU and then remap
 * the underlying physical memory into a fixed virtual location in KSEG2,
 * much like we do with the u area.
 *
 * The data structures can be accessed in one of two way: by exlicit index
 * and reference to the per-CPU page (as a virtual address) or by reference
 * to the fixed virtual address in KSEG2.
 *
 * i386: Per-CPU data is 5 pages.  The boot CPU's data must be allocated
 * statically because it is used by the startup code.  All others CPUs' data
 * is allocated dynamically.  Each CPU's data is mapped to a fixed virtual
 * address after the U-block (using the same per-CPU page table as the 
 * U-block).
 */

#include "sys/types.h"		/* cause of immu.h */
#include "sys/param.h"		/* to get CPUPRISIZE */
#include "sys/immu.h"		/* to get NBPP */
#ifndef _SYS_PERCPU_H
#include "sys/percpu.h"		/* to get percpu */
#endif
#include "sys/sysinfo.h"	/* to get sysinfo etc */
#include "sys/cred.h"		/* because dnlc.h needs it */
#include "sys/vnode.h"		/* because dnlc.h needs it */
#include "sys/dnlc.h"		/* to get ncstats */
#include "vm/seg_map.h"		/* to get segmapcnt */
#ifdef mips
#include "vm/page.h"
#include "vm/vm_hat.h"
#endif
#ifdef u3b2
#include "sys/sbd.h"
#include "sys/psw.h"
#include "sys/pcb.h"
#endif
#ifdef 	__i386__
#include "sys/pic.h"
#include "sys/seg.h"
#endif /* __i386__ */

#ifdef mips

#define PRIVSIZE	CPUPRISIZE	/* Size of per-CPU data in pages */

#endif /* mips */

#ifdef	__i386__

#define	PRIVLOC		0xe0020000	/* Address of virtual window */
#define	PRIVSIZE	5		/* Size of per-CPU data in pages */

#endif /* __i386__ */

#ifdef u3b2

#define	PRIVLOC		0x100000	/* Address of virtual window */
#define	PRIVSIZE	13		/* Size of per-CPU data in pages */

#endif

typedef  union	cpupriv {
	struct	cp_data {
#ifdef __i386__
/* These entries are first because they must be page-aligned. */
		pte_t   cp_d_kpd0[NPGPT];	/* Page table directory */
		pte_t   cp_d_usrtbl[NPGPT];	/* U block page table */
		char    cp_d_ovrstk[NBPP];	/* Kernel stack overflow page */
		pte_t   cp_d_stktbl[NPGPT];	/* Overflow stack page table */
/* End of page-aligned entries. */

/*
 * The GDT and IDT are in cpupriv (not percpu) to make initialization easier.
 * They are declared here in compile-time format.
 */
		struct seg_desc  cp_d_gdt[GDTSZ]; /* global descriptor table */
						  /* (180 words) */
		struct gate_desc cp_d_idt[IDTSZ]; /* interrupt descriptor */
						  /* table (256 words) */
		int	pad[4];			  /* align to 32 bytes */
#endif

#ifdef u3b2
		/* kernel entry pcbs */
		struct kpcb cp_d_kpcb_null;
		struct kpcb cp_d_kpcb_px;
		struct kpcb cp_d_kpcb_sx;
		struct kpcb cp_d_kpcb_pswtch;
		struct kpcb cp_d_kpcb_qrun;
		struct kpcb cp_d_kpcb_L9;
		struct kpcb cp_d_kpcb_L15;
		struct kpcb cp_d_kpcb_nmi;
#endif

		/* Per-CPU data defined in percpu.h */
		struct	percpu	cp_d_percpu;

/* Miscellaneous statistics are made per-CPU to avoid cache contention. */
		struct	sysinfo	cp_d_sysinfo;
		struct  syserr  cp_d_syserr;
		struct  vminfo  cp_d_vminfo;    /* VM stats */
		struct  rtminfo cp_d_rtminfo;   /* Real Time stats */
		struct  minfo   cp_d_minfo;
		struct	ncstats cp_d_ncstats;	/* name cache stats */
		struct	syswait cp_d_syswait;
		struct	segmapcnt cp_d_segmapcnt; /* segmap stats */
#ifdef mips
		/*
		 * Accounting for the 'C' level tlbmiss/mod
		 * activity.
		 */
		struct	tlbmiss_stats cp_d_tlbmiss_stats;
		struct	tlbmod_stats  cp_d_tlbmod_stats;
		struct	tlbflush_stats cp_d_tlbflush_stats;
		/*
		 * Accounting for the machine code tlbmiss
		 * activity (where most of the action is).
		 */
		unsigned cp_d_utlbmisses;
		unsigned cp_d_doublemisses;
		unsigned cp_d_ktlbmisses;
#ifdef B20R
		unsigned short cp_d_iplppl;
#endif /* B20R */
#endif

#ifdef u3b2
		/* kernel stacks */
		int cp_d_kstk8[QSTKSZ];
		int cp_d_stackseparator1;
		int cp_d_kstkA[ISTKSZ];
		int cp_d_stackseparator2;
		int cp_d_kstkF[ISTKSZ];
		int cp_d_stackseparator3;
		int cp_d_kstkQR[QSTKSZ];
		int cp_d_stackseparator4;
		int cp_d_kstkSX[ISTKSZ];
		int cp_d_stackseparator5;
		int cp_d_kstknmi[ISTKSZ];
		int cp_d_stackseparator6;

		/* kernel interrupt stack, and during initialization, isp[0]
		 * contains the physical address of the sdt for section 0
		 * for each MPB */
		pcb_t *cp_d_krnl_isp[19];
#endif
	} cp_data;
	char	cp_page[PRIVSIZE*NBPP];
} cpupriv_t;

#ifdef _KERNEL

extern	cpupriv_t	cpupriv;


/* The following variables are labels that are ".set" in
 * a ml file to point to the appropriate entry in the 
 * percpu structure.
 */

#ifndef u3b2v4
#ifdef __STDC__

extern  int		xclbind;
#ifndef lint
#pragma large_data	xclbind
#endif
extern	struct proc	*idleproc;
#ifndef lint
#pragma large_data	idleproc
#endif
extern	cpuid_t		lcpunum;
#ifndef lint
#pragma	large_data	lcpunum
#endif
extern	int		lcpumask;
#ifndef lint
#pragma	large_data	lcpumask
#endif
extern	int		cpu_revid;
#ifndef lint
#pragma	large_data	cpu_revid
#endif
extern	int		runrun;
#ifndef lint
#pragma large_data	runrun
#endif
extern	int		kprunrun;
#ifndef lint
#pragma large_data	kprunrun
#endif
extern	int		npwakecnt;
#ifndef lint
#pragma large_data	npwakecnt
#endif
extern	cpumask_t	migrate_from;
#ifndef lint
#pragma large_data	migrate_from
#endif
extern	int		curpri;
#ifndef lint
#pragma large_data	curpri
#endif
extern	short		intrcxt;
#ifndef lint
#pragma large_data	intrcxt
#endif
extern	char		locktestoff;
#ifndef lint
#pragma large_data	locktestoff
#endif
#ifdef _SHAREII
extern	char		ShareCpuFlag;
#ifndef lint
#pragma large_data	ShareCpuFlag
#endif
#endif /* _SHAREII */
extern	signed char	lastlock;
#ifndef lint
#pragma large_data	lastlock
#endif
extern	struct mplock	*locklist[MAXLOCKED];
#ifndef lint
#pragma large_data	locklist
#endif
extern	signed char	lastlock_ddi;
#ifndef lint
#pragma large_data	lastlock_ddi
#endif
extern	struct mplock	*locklist_ddi[MAXLOCKED];
#ifndef lint
#pragma large_data	locklist_ddi
#endif
extern	int		backoff;
#ifndef lint
#pragma large_data	backoff
#endif
extern	int		basepriority;
#ifndef lint
#pragma large_data	basepriority
#endif
extern	unsigned	idleadj;
#ifndef lint
#pragma large_data	idleadj
#endif
extern	struct	proc	*fp_proc;
#ifndef lint
#pragma large_data	fp_proc
#endif
extern	struct	proc	*curproc;
#ifndef lint
#pragma large_data	curproc
#endif
extern	struct	proc	*old_curproc;
#ifndef lint
#pragma large_data	old_curproc
#endif
extern	int	idleflag;
#ifndef lint
#pragma large_data	idleflag
#endif
extern	uint	cpu_tlb_gncnt;
#ifndef lint
#pragma large_data	cpu_tlb_gncnt
#endif
extern	uint	dqgen;
#ifndef lint
#pragma large_data	dqgen
#endif
extern	int	dotimein;
#ifndef lint
#pragma large_data	dotimein
#endif

#ifdef mips
extern	int	dosoftint;
#ifndef lint
#pragma large_data	dosoftint
#endif
extern	int	tlbpid_assigned;
#ifndef lint
#pragma large_data	tlbpid_assigned
#endif
extern	int	tlbpid_gen;
#ifndef lint
#pragma large_data	tlbpid_gen
#endif
extern	volatile uint	curipl;
#ifndef lint
#pragma large_data	curipl
#endif
extern	volatile uint	maxqueipl;
#ifndef lint
#pragma large_data	maxqueipl
#endif
extern	volatile ulong	iplquemask;
#ifndef lint
#pragma large_data	iplquemask
#endif
extern	uint		iplfreecnt;
#ifndef lint
#pragma large_data	iplfreecnt
#endif
extern	uint		iplchoke;
#ifndef lint
#pragma large_data	iplchoke
#endif
extern	struct iplelem	*iplfree;
#ifndef lint
#pragma large_data	iplfree
#endif
extern	struct iplque	*iplbase;
#ifndef lint
#pragma large_data	iplbase
#endif
extern	int		unix_tick;
#ifndef lint
#pragma large_data	unix_tick
#endif
extern	uint		fast_tod_lo;
#ifndef lint
#pragma	large_data	fast_tod_lo
#endif
extern	uint		tick_tod_hi;
#ifndef lint
#pragma	large_data	tick_tod_hi
#endif
extern	uint		tick_tod_lo;
#ifndef lint
#pragma	large_data	tick_tod_lo
#endif
extern	uint		slow_tod_hi;
#ifndef lint
#pragma	large_data	slow_tod_hi
#endif
extern	uint		slow_tod_lo;
#ifndef lint
#pragma	large_data	slow_tod_lo
#endif
extern	uint		fast_offset;
#ifndef lint
#pragma	large_data	fast_offset
#endif
extern	uint		itmr_ckpt_cnt;
#ifndef lint
#pragma	large_data	itmr_ckpt_cnt
#endif
extern	uint		rtod_ckpt_cnt;
#ifndef lint
#pragma	large_data	rtod_ckpt_cnt
#endif
extern	int		tod_hi_diff;
#ifndef lint
#pragma	large_data	tod_hi_diff
#endif
extern	char		clk_speed;
#ifndef lint
#pragma	large_data	clk_speed
#endif
#ifdef _SMP
extern	char		hb_cpu;
#ifndef lint
#pragma	large_data	hb_cpu
#endif
extern	char		hb_cnt;
#ifndef lint
#pragma	large_data	hb_cnt
#endif
extern	uint		hb_value;
#ifndef lint
#pragma	large_data	hb_value
#endif
#endif /* _SMP */
extern	int		delay_mult;
#ifndef lint
#pragma	large_data	delay_mult
#endif
extern	unsigned	utlbmisses;
#ifndef lint
#pragma large_data	utlbmisses
#endif
extern	unsigned	doublemisses;
#ifndef lint
#pragma large_data	doublemisses
#endif
extern	unsigned	ktlbmisses;
#ifndef lint
#pragma large_data	ktlbmisses

#if defined(SNI_R4000) && defined(_SMP)

#ifdef B20R
extern	unsigned short 	iplppl;
#pragma large_data	iplppl
#endif /* B20R */

extern	uint		si_cache_size;
#pragma	large_data	si_cache_size
extern	uint		si_inx_mask;
#pragma	large_data	si_inx_mask
extern	uint		sd_cache_size;
#pragma	large_data	sd_cache_size
extern	uint		sd_inx_mask;
#pragma	large_data	sd_inx_mask
#endif /* SNI_R4000 && _SMP */

extern	uint		icache_size;
#pragma	large_data	icache_size
extern	uint		dcache_size;
#pragma	large_data	dcache_size
#endif
extern	struct ncstats	ncstats;
#ifndef lint
#pragma large_data	ncstats
#endif
extern	struct rtminfo	rtminfo;
#ifndef lint
#pragma large_data	rtminfo
#endif
#endif	/* mips */

#ifdef __i386__
extern  void            (*cmn_errfunc)();
extern	void		(*nmi_func)();
extern  lck_t           linkfunclock;
extern  INT_FUNC        *intfuncpntr;
extern  INT_FUNC        *intfuncavail;
extern  INT_FUNC        *oldest_func;
extern	int		sberrcount;
extern	ulong		cr_regs[];
#endif

#endif /* __STDC__ */

#else /* u3b2v4 */

#ifdef __STDC__

extern  int		xclbind;
extern	struct proc	*idleproc;
extern	cpuid_t		lcpunum;
extern	int		lcpumask;
extern	int		runrun;
extern	int		kprunrun;
extern	int		npwakecnt;
extern	cpumask_t	migrate_from;
extern	int		curpri;
extern	short		intrcxt;
extern	char		locktestoff;
extern	signed char	lastlock;
extern	struct mplock	*locklist[MAXLOCKED];
extern	signed char	lastlock_ddi;
extern	struct mplock	*locklist_ddi[MAXLOCKED];
extern	int		backoff;
extern	int		basepriority;
extern	unsigned	idleadj;
extern	struct	proc	*fp_proc;
extern	struct	proc	*curproc;
extern	struct	proc	*old_curproc;
extern	int	idleflag;
extern	uint	lastdqgen;
extern	uint	cpu_tlb_gncnt;
extern	uint	dqgen;
extern	int	dotimein;
extern	int	dosoftint;
extern	int	tlbpid_assigned;
extern	int	tlbpid_gen;
extern	volatile uint	curipl;
extern	volatile uint	maxqueipl;
extern	volatile ulong	iplquemask;
extern	struct iplelem	*iplfree;
extern	struct iplque	*iplbase;

#endif /* __STDC__ */

#endif /* u3b2v4 */

extern	cpupriv_t 	*cpuprivvir[];
				/* array of pointers to virtual base of cpu 
				 * private pages. used for indexed access 
				 * between cpus */

/* 
 * Generic macros for non-local access of cpupriv elements
 */

#define PERCPU(i, x)	(cpuprivvir[(i)]->cp_data.cp_d_percpu.x)
#define CPUPRIV(i, x)	(cpuprivvir[(i)]->cp_data.x)
#define SYSINFO(i, x)	(cpuprivvir[(i)]->cp_data.cp_d_sysinfo.x)
#define MINFO(i, x)	(cpuprivvir[(i)]->cp_data.cp_d_minfo.x)
#define VMINFO(i, x)	(cpuprivvir[(i)]->cp_data.cp_d_vminfo.x)
#define NCSTATS(i, x)	(cpuprivvir[(i)]->cp_data.cp_d_ncstats.x)
#define SYSWAIT(i, x)	(cpuprivvir[(i)]->cp_data.cp_d_syswait.x)
#define SEGMAPCNT(i, x)	(cpuprivvir[(i)]->cp_data.cp_d_segmapcnt.x)

#ifdef	mips
/* get tlbpid for current cpu: needs vm_hat.h */

#ifdef DEBUG
#define	CURCPUPID(hatp) (ASSERT((hatp)->hat_data->hd_tlbpid.ht_gen[lcpunum] == tlbpid_gen), (hatp)->hat_data->hd_tlbpid.ht_tlbpid[lcpunum])
#else
#define	CURCPUPID(hatp)	((hatp)->hat_data->hd_tlbpid.ht_tlbpid[lcpunum])
#endif /* DEBUG */
#endif /* mips */

#endif /* _KERNEL */

#endif	/* SYS_CPUPRIV_H */
