#ident "@(#)$Header: ufs_inode.h,v 1.7 92/10/09 10:38:35 svr4 Exp $ SNI"
/*
 * "@(#)ufs_inode.h	Revision: 3.1	92/03/11 20:22:27"
 */

/*	Copyright (c) 1984, 1986, 1987, 1988, 1989 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _SYS_FS_UFS_INODE_H
#define _SYS_FS_UFS_INODE_H

/*
 * The I node is the focus of all local file activity in UNIX.
 * There is a unique inode allocated for each active file,
 * each current directory, each mounted-on file, each mapping,
 * and the root.  An inode is `named' by its dev/inumber pair.
 * Data in icommon is read in from permanent inode on volume.
 */

#define EFT_MAGIC 0x90909090	/* magic cookie for EFT */
#define	NDADDR	12		/* direct addresses in inode */
#define	NIADDR	3		/* indirect addresses in inode */

struct inode {
	struct	inode *i_chain[2];	/* must be first */
	struct	vnode i_vnode;	/* vnode associated with this inode */
	struct  ufsvfs *i_ufsvfsp;	/* filesystem the inode belongs to */
	struct	vnode *i_devvp;	/* vnode for block I/O */
	u_int	i_flag;		/* protected by IWLOCKED or IRLOCKED 
				 * ##jk by i_mplock for now (see NOTE below). */
	u_int	i_xflag;	/* protected by i_mplock */
	dev_t	i_dev;		/* device where inode resides */
	ino_t	i_number;	/* i number, 1-to-1 with device address */
	off_t	i_diroff;	/* offset in dir, where we found last entry */
	struct	fs *i_fs;	/* file sys associated with this inode */
	struct	dquot *i_dquot;	/* quota structure controlling this file */
	int	i_count;	/* number of inode locks for i_owner */
	short	i_owner;	/* proc index of process locking inode */
	short	i_rwowner;	/* proc index of process holding rwlock */
	off_t	i_nextw;	/* next byte write offset (write-behind) */
	off_t	i_startwb;	/* where to start doing write-behind */
	daddr_t	i_nextr;	/* next byte read offset (read-ahead) */
	daddr_t	i_lastrhead;	/* last read-ahead */
	int	i_readahead;	/* read-ahead count */
	struct inode  *i_freef;	/* free list forward */
	struct inode **i_freeb;	/* free list back */
	ulong	i_vcode;	/* version code attribute */
	long	i_mapcnt;	/* mappings to file pages */
	int	*i_map;		/* block list for the corresponding file */
	lck_t	i_mplock;	/* inode spinlock */
	struct buf *i_giobp;	/* buffer header for group io */
#ifdef mips
	uint	i_spare[4];	/* pad for faster copies */
#endif
	struct 	icommon {
		o_mode_t ic_smode;	/*  0: mode and type of file */
		short	ic_nlink;	/*  2: number of links to file */
		o_uid_t	ic_suid;		/*  4: owner's user id */
		o_gid_t	ic_sgid;		/*  6: owner's group id */
		quad	ic_size;	/*  8: number of bytes in file */
#ifdef _KERNEL
		struct timeval ic_atime;/* 16: time last accessed */
		struct timeval ic_mtime;/* 24: time last modified */
		struct timeval ic_ctime;/* 32: last time inode changed */
#else
		time_t	ic_atime;	/* 16: time last accessed */
		long	ic_atspare;
		time_t	ic_mtime;	/* 24: time last modified */
		long	ic_mtspare;
		time_t	ic_ctime;	/* 32: last time inode changed */
		long	ic_ctspare;
#endif
		daddr_t	ic_db[NDADDR];	/* 40: disk block addresses */
		daddr_t	ic_ib[NIADDR];	/* 88: indirect blocks */
		long	ic_flags;	/* 100: status, currently unused */
		long	ic_blocks;	/* 104: blocks actually held */
		long	ic_gen;		/* 108: generation number */
		mode_t	ic_mode;	/* 112: EFT version of mode*/
		uid_t	ic_uid;		/* 116: EFT version of uid */
		gid_t	ic_gid;		/* 120: EFT version of gid */
		ulong	ic_eftflag;	/* 124: indicate EFT version*/

	} i_ic;
};

struct dinode {
	union {
		struct	icommon di_icom;
		char	di_size[128];
	} di_un;
};

#define i_mode		i_ic.ic_mode
#define	i_nlink		i_ic.ic_nlink
#define i_uid		i_ic.ic_uid
#define i_gid		i_ic.ic_gid
#define i_smode		i_ic.ic_smode
#define i_suid		i_ic.ic_suid
#define i_sgid		i_ic.ic_sgid
#define i_eftflag	i_ic.ic_eftflag

/* ugh! -- must be fixed */
#if defined(__vax__) || defined(__i386__)
#define	i_size		i_ic.ic_size.val[0]
#endif
#if defined(__mc68000__) || defined(__sparc__) || defined(u3b2) || defined(u3b15) || defined(mips)
#define	i_size		i_ic.ic_size.val[1]
#endif
#define	i_db		i_ic.ic_db
#define	i_ib		i_ic.ic_ib

#define	i_atime		i_ic.ic_atime
#define	i_mtime		i_ic.ic_mtime
#define	i_ctime		i_ic.ic_ctime

#define i_flags		i_ic.ic_flags
#define i_blocks	i_ic.ic_blocks
#define	i_oldrdev	i_ic.ic_db[0]
#define	i_rdev		i_ic.ic_db[1]
#define	i_gen		i_ic.ic_gen
#define	i_forw		i_chain[0]
#define	i_back		i_chain[1]

#define di_ic		di_un.di_icom
#define	di_mode		di_ic.ic_mode
#define	di_nlink	di_ic.ic_nlink
#define	di_uid		di_ic.ic_uid
#define	di_gid		di_ic.ic_gid
#define di_smode	di_ic.ic_smode
#define di_suid		di_ic.ic_suid
#define di_sgid		di_ic.ic_sgid
#define di_eftflag	di_ic.ic_eftflag

#if defined(__vax__) || defined(__i386__)
#define	di_size		di_ic.ic_size.val[0]
#endif
#if defined(__mc68000__) || defined(__sparc__) || defined(u3b2) || defined(u3b15) || defined(mips)
#define	di_size		di_ic.ic_size.val[1]
#endif
#define	di_db		di_ic.ic_db
#define	di_ib		di_ic.ic_ib

#define	di_atime	di_ic.ic_atime
#define	di_mtime	di_ic.ic_mtime
#define	di_ctime	di_ic.ic_ctime

#define	di_oldrdev	di_ic.ic_db[0]
#define	di_rdev		di_ic.ic_db[1]
#define	di_flags	di_ic.ic_flags
#define	di_blocks	di_ic.ic_blocks
#define	di_gen		di_ic.ic_gen

/*
 * Definition for Embedded Symbolic Links.
 * Symbolic links smaller than MAXEMBLINK are embedded in the inode over the
 * the fields for block addresses using the following definition.  The i_size
 * in the inode is set to zero to indicate that the link is embedded.
 */
#define MAXEMBLINK	(NDADDR + NIADDR - 1) * 4
struct emblink {
	ulong 	e_len;			/* length of symbolic link. */
	char	e_link[MAXEMBLINK];	/* contents of symbolic link. */
};
#define I_EMBLINK(ip)	(struct emblink *)(ip)->i_ic.ic_db
#define DI_EMBLINK(ip)	(struct emblink *)(ip)->di_ic.ic_db

/* flags */
#define	Iunused0	0x001		/* was ILOCKED bit (MOVED to xflag) */
#define	IUPD		0x002		/* file has been modified */
#define	IACC		0x004		/* inode access time to be updated */
#define	IMOD		0x008		/* inode has been modified */
#define	Iunused1	0x010		/* was IWANT bit (MOVED to xflag) */
#define	ISYNC		0x020		/* do all allocation synchronously */
#define	ICHG		0x040		/* inode has been changed */
#define	Iunused2	0x080		/* was file lock wait -- not used */
#define	Iunused3	0x100		/* was IREF bit (MOVED to xflag) */
#define	INOACC		0x200		/* no access time update in getpage */
#define	IMODTIME	0x400		/* mod time already set */
#define	Iunused4	0x1000		/* was IRWLOCKED bit (MOVED to xflag) */

/* xflag bits */
#define	IWLOCKED	0x001		/* inode is (write) locked */
#define	IRLOCKED	0x002		/* inode is (read) locked */
#define	INOLOCKED	(IWLOCKED|IRLOCKED)
#define	IWANTED		0x004		/* someone waiting on write lock */
#define	IRWANTED	0x008		/* someone waiting on read lock */ 
#define	INOREF		0x010		/* some process waiting on lock */
#define	INTRANS		0x020		/* disk read of icommon in progress */
#define	IRWLOCKED	0x040		/* deadlock prevention lock */
#define	IRWWANT	0x080		/* want "" lock (see below) */
#define IINACTIVE	0x100		/* iinactive in progress */

/* modes */
#define	IFMT		0170000		/* type of file */
#define	IFIFO		0010000		/* named pipe (fifo) */
#define	IFCHR		0020000		/* character special */
#define	IFDIR		0040000		/* directory */
#define	IFBLK		0060000		/* block special */
#define	IFREG		0100000		/* regular */
#define	IFLNK		0120000		/* symbolic link */
#define	IFSOCK		0140000		/* socket */

#define	ISUID		04000		/* set user id on execution */
#define	ISGID		02000		/* set group id on execution */
#define	ISVTX		01000		/* save swapped text even after use */
#define	IREAD		0400		/* read, write, execute permissions */
#define	IWRITE		0200
#define	IEXEC		0100


#ifdef _KERNEL
struct inode *ufs_inode;		/* the inode table itself */
struct inode *inodeNINODE;		/* the end of the inode table */
extern int ufs_ninode;			/* number of slots in the table */

extern struct vnodeops ufs_vnodeops;	/* vnode operations for ufs */

extern ino_t dirpref();
extern daddr_t blkpref();

/*
 * Convert between inode pointers and vnode pointers
 */
#define VTOI(VP)	((struct inode *)(VP)->v_data)
#define ITOV(IP)	((struct vnode *)&(IP)->i_vnode)

#ifdef _SMP
#include "sys/types.h"			/* for lck_t */
#include "sys/mplocks.h"		/* for mplock interface */
#endif
#include "sys/proc.h"			/* for SSWLOCKS */
#include "sys/cpupriv.h"		/* for curproc */
#include "sys/debug.h"			/* for ASSERTs */
#include "sys/fs/ufs_statis.h"		/* for statistics */

/*
 * Lock and unlock inodes.
 */
/*
 * IRWLOCK is to prevent a deadlock situation where:
 *	process A   opens file 1, and mmaps file 2
 *	process B   opens file 2, and mmaps file 1
 *
 *	process A   then reads file 1 into the mmapped space for file 2
 *	process B   then reads file 2 into the mmapped space for file 1
 *
 *	deadlock
 *
 * NOTE: all of these macros expect "ip->i_mplock" to be locked
 */
#define	IRWLOCK(ip) { \
	ASSERTLOCKED(&(ip)->i_mplock); \
	while ((ip)->i_xflag & IRWLOCKED) { \
		(ip)->i_xflag |= IRWWANT; \
		(void) SLEEPFREE1((caddr_t)(&ip->i_xflag), PINOD, &(ip)->i_mplock); \
		UFSINCR(ufs_irwsleep, 1); \
		PSPIN(&(ip)->i_mplock, &mpsbuf[MPS_SPIN_UFS_INODE]); \
	} \
	(ip)->i_rwowner = curproc->p_slot; \
	(ip)->i_xflag |= IRWLOCKED; \
	if (((ip)->i_vnode.v_flag & VISSWAP) != 0){ \
		int smplvl; \
		curproc->p_swlocks++;	\
		SPLHI(smplvl);	\
		PSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
		curproc->p_flag |= SSWLOCKS;\
		VSPIN(&curproc->p_mplock, STAT_P_FLAG);			\
		SPLX(smplvl);	\
	} \
}

#define	IRWUNLOCK(ip) { \
	ASSERTLOCKED(&(ip)->i_mplock); \
	ASSERT((ip)->i_xflag & IRWLOCKED); \
	if (((ip)->i_vnode.v_flag & VISSWAP) != 0) \
		if (--curproc->p_swlocks == 0) { \
			int smplvl;	\
			SPLHI(smplvl);	\
			PSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
			curproc->p_flag &= ~SSWLOCKS; \
			VSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
			SPLX(smplvl);	\
		} \
	(ip)->i_xflag &= ~IRWLOCKED; \
	if ((ip)->i_xflag & IRWWANT) { \
		if (wakeup1((caddr_t)(&ip->i_xflag)) == 0) \
			(ip)->i_xflag &= ~IRWWANT; \
	} \
}

/*
 * ILOCK - write lock an inode
 */
#define	ILOCK(ip) { \
	ASSERTLOCKED(&(ip)->i_mplock); \
	while (((ip)->i_xflag & IWLOCKED) && \
	    (ip)->i_owner != curproc->p_slot) { \
		(ip)->i_xflag |= IWANTED; \
		(void) SLEEPFREE1((caddr_t)(ip), PINOD,  &(ip)->i_mplock); \
		UFSINCR(ufs_isleep, 1); \
		PSPIN(&(ip)->i_mplock, &mpsbuf[MPS_SPIN_UFS_INODE]); \
	} \
	(ip)->i_owner = curproc->p_slot; \
	(ip)->i_count++; \
	(ip)->i_xflag |= IWLOCKED; \
	if (((ip)->i_vnode.v_flag & VISSWAP) != 0){ \
		int smplvl; \
		curproc->p_swlocks++;	\
		SPLHI(smplvl);	\
		PSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
		curproc->p_flag |= SSWLOCKS;\
		VSPIN(&curproc->p_mplock, STAT_P_FLAG);			\
		SPLX(smplvl);	\
	} \
}

/*
 * IRLOCK - read lock an inode
 */
#define	IRLOCK	ILOCK		/* ## for now readlocks are writelocks */

/*
 * ILOCKED - check whether the inode is locked in this context.
 */
#define ILOCKED(ip)	\
	(((ip)->i_xflag & IWLOCKED) && ((ip)->i_owner == curproc->p_slot))

/*
 * I_RWLOCKED - check whether the inode is locked for read/write in this
 * context.
 */
#define I_RWLOCKED(ip)	\
	(((ip)->i_xflag & IRWLOCKED) && ((ip)->i_rwowner == curproc->p_slot))

/*
 * IUNLOCK - unlock an inode (either read or write locked)
 */
#define	IUNLOCK(ip) { \
	ASSERTLOCKED(&(ip)->i_mplock); \
	if (--(ip)->i_count < 0) \
		cmn_err(CE_PANIC, "IUNLOCK"); \
	if (((ip)->i_vnode.v_flag & VISSWAP) != 0) \
		if (--curproc->p_swlocks == 0) { \
			int smplvl; \
			SPLHI(smplvl);	\
			PSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
			curproc->p_flag &= ~SSWLOCKS; \
			VSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
			SPLX(smplvl);	\
		} \
	if ((ip)->i_count == 0) { \
		(ip)->i_xflag &= ~IWLOCKED; \
		if ((ip)->i_xflag & IWANTED) { \
			if (((ip)->i_vnode.v_flag & VISSWAP) == 0) { \
				if (wakeup1((caddr_t)(ip)) == 0) \
					(ip)->i_xflag &= ~IWANTED; \
			} else { \
				(void) wakeprocs((caddr_t)(ip), PRMPT); \
				(ip)->i_xflag &= ~IWANTED; \
			} \
		} \
	} \
}

/*
 * Update inode if changed.  Requires ILOCK.
 */
#define IUPDAT(ip, waitfor) { \
	if (ip->i_flag & (IUPD|IACC|ICHG|IMOD)) \
		ufs_iupdat(ip, waitfor); \
}

/*
 * Mark an inode with the current (unique) timestamp.
 * ##_SMP Must be called with a inode spinlock held.
 *	  NOTE:  The i_flag field should really be protected by ILOCK.
 *		 However, with the current structuring of code it is a lot
 *		 easier (and perhaps even more efficient) to protect this
 *		 with the spinlock.
 */
#define IMARK(ip) 	ufs_imark(ip)

#define ITIMES(ip) { \
	if ((ip)->i_flag & (IUPD|IACC|ICHG)) { \
		(ip)->i_flag |= IMOD; \
		IMARK(ip); \
		(ip)->i_flag &= ~(IACC|IUPD|ICHG); \
	} \
}

/*
 * Allocate the specified block in the inode
 * and make sure any in-core pages are initialized.
 */
#define	BMAPALLOC(ip, lbn, size) \
	ufs_bmap((ip), (lbn), (daddr_t *)NULL, (daddr_t *)NULL, (size), S_WRITE, 0)

/*
 * Macros and enum to use iget for cases in which the inode should
 * be returned with xflag locked (iget) or with xflag unlocked but
 * with the semaphore locked (igetx).  Note that the name defintions
 * don't quite match up with the way we do this in other places but
 * we need to preseve the "iget" name.
 */
enum iget_op { SETXLOCK, SETMPLOCK };

#define ufs_iget(vfsp, fs, ino, ipp) \
			ufs_igetc(vfsp, fs, ino, ipp, SETXLOCK)
#define ufs_igetx(vfsp, fs, ino, ipp) \
			ufs_igetc(vfsp, fs, ino, ipp, SETMPLOCK)

#define ESAME	(-1)		/* trying to rename linked files (special) */

/*
 * Check that file is owned by current user or user is su.
 */
#define OWNER(CR, IP)	(((CR)->cr_uid == (IP)->i_uid)? 0: (suser(CR)? 0: EPERM))

#define	UFS_HOLE	(daddr_t)-1	/* value used when no block allocated */

/*
 * enums
 */
enum de_op	{ DE_CREATE, DE_MKDIR, DE_LINK, DE_RENAME };	/* direnter ops */
enum dr_op	{ DR_REMOVE, DR_RMDIR, DR_RENAME };	/* dirremove ops */

/*
 * This overlays the fid structure (see vfs.h)
 */
struct ufid {
	u_short	ufid_len;
	ino_t	ufid_ino;
	long	ufid_gen;
};

/*
 * UFS VFS private data.
 * ##: should be in ufs_fs.h
 * ##SMP The quota related fields are synchronized using quota_mplock.  
 */
struct ufsvfs {
	struct vnode	*vfs_root;	/* root vnode */
	struct fs	*vfs_fsp;	/* buffer containing superblock */
	struct vnode	*vfs_devvp;	/* block device vnode */
	struct inode	*vfs_qinod;	/* QUOTA: pointer to quota file */
	u_short		vfs_qflags;	/* QUOTA: filesystem flags */
	u_short		vfs_ufsflags;	/* flags for filesystem state */
	u_long		vfs_btimelimit;	/* QUOTA: block time limit */
	u_long		vfs_ftimelimit;	/* QUOTA: file time limit */
	lck_t		vfs_ino_mplock;	/* %%SMP: lock for inode allocation */
	lck_t		vfs_blk_mplock;	/* %%SMP: lock for block allocation */
	struct vfs	*vfs_vfsp;	/* generic part */
	struct ufsvfs	*vfs_nextp;	/* forms list of ufs filesystems */
};

/* Defines for vfs_ufsflags in the ufsvfs struct. */
#define UFS_INOLOCKED		0x0001	/* Filesystem has locked inodes */
#define UFS_INCONSISTENT	0x0002	/* Filesystem has inconsistency */

/* Mount list */
extern struct ufsvfs *ufslist;
extern lck_t ufslist_mplock;

#ifdef _SMP
/* Macros for locking inode and block allocation information. */
#define	UFS_INOLOCK(ip)		\
	PSPIN(&(ip)->i_ufsvfsp->vfs_ino_mplock, &mpsbuf[MPS_SPIN_UFS_ALLOC])
#define	UFS_INOUNLOCK(ip)	\
	VSPIN(&(ip)->i_ufsvfsp->vfs_ino_mplock, &mpsbuf[MPS_SPIN_UFS_ALLOC])
#define	UFS_BLKLOCK(ip)		\
	PSPIN(&(ip)->i_ufsvfsp->vfs_blk_mplock, &mpsbuf[MPS_SPIN_UFS_ALLOC])
#define	UFS_BLKUNLOCK(ip)	\
	VSPIN(&(ip)->i_ufsvfsp->vfs_blk_mplock, &mpsbuf[MPS_SPIN_UFS_ALLOC])
#else /* _SMP */
#define	UFS_INOLOCK(fs)
#define	UFS_INOUNLOCK(fs)
#define	UFS_BLKLOCK(fs)
#define	UFS_BLKUNLOCK(fs)
#endif /* _SMP */

#endif
#endif /* _SYS_FS_UFS_INODE_H */
