#ident "@(#)$Header: xnamnode.h,v 1.5 92/10/09 10:38:38 svr4 Exp $ SNI"
/*
 * "@(#)xnamnode.h	Revision: 3.1	92/03/11 20:22:31"
 */

/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990, 1991 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _FS_XNAMNODE_H
#define _FS_XNAMNODE_H

/*#ident	"@(#)ATTDSG gencpu/gensys/uts/sys/fs/xnamnode.h	1.1.1.2"*/

#ifdef __i386__
#define _FSCOMPAT
#endif

/* ##SMP: For compatibility locking - see below */

#include "sys/user.h"
#include "sys/param.h"
#include "sys/systm.h"
#include "sys/mplocks.h"
#include "sys/mpinline.h"

/*
 * The xnamnode represents a special XENIX file in any filesystem.  There is
 * one xnamnode for each active special XENIX file.  Filesystems that support
 * special files use xnamvp(vp, dev, type, cr) to convert a normal
 * vnode to a special vnode in the ops lookup() and create().
 *
 */

struct xsem {			/* XENIX semaphore */
	short  x_scount;	/* current semaphore count */
	short  x_eflag;		/* err flg */
	struct file *x_headw;	/* first waiter */
	struct file *x_tailw;	/* last waiter */
};

struct xsd {               /* XENIX shared data */
	union {
		struct xsd *x_chain; /* next available shared data structure */
		struct anon_map *xamp;  /* Pointer to shared data segment */
	} x_sun;
	unsigned x_len;	     /* limit of segment (seg size - 1) */
	short    x_snum;         /* serial # for sdgetv, sdwaitv */
	short    x_flags;        /* LOCKED, etc. */
}; 


struct xnamnode {
	struct	xnamnode *x_next;		/* must be first */
	struct	vnode x_vnode;		/* vnode associated with this xnamnode */
	struct	vnode *x_realvp;	/* vnode for the fs entry (if any) */
	ushort	x_flag;			/* flags, see below */
	dev_t	x_dev;			/* device the xnamnode represents */
	dev_t	x_fsid;			/* file system identifier */
	daddr_t	x_nextr;		/* next byte read offset (read-ahead) */
	long	x_size;			/* block device size in bytes */
	time_t  x_atime;		/* time of last access */
	time_t  x_mtime;		/* time of last modification */
	time_t  x_ctime;		/* time of last attributes change */
	int	x_count;		/* count of opened references */
	mode_t	x_mode;			/* file mode and type */
	uid_t	x_uid;			/* owner */
	gid_t	x_gid;			/* group */
	union x_u {
		struct  xsem *xsem;	/* ptr to XENIX semaphores */
		struct  xsd *xsd;	/* ptr to XENIX semaphores */
	} x_un;
	struct proc *x_powns;		/* XXX vm debugging */
};

#define x_sem	x_un.xsem	/* for v_type==VXNAM && v_rdev==XNAM_SEM */
#define x_sd	x_un.xsd	/* for v_type==VXNAM && v_rdev==XNAM_SD */

/* flags */
#define XNAMLOCKED		0x01		/* xnamnode is locked */
#define XNAMUPD		0x02		/* update device access time */
#define XNAMACC		0x04		/* update device modification time */
#define XNAMWANT		0x10		/* some process waiting on lock */
#define XNAMCHG		0x40		/* update device change time */

/* XENIX semaphore sub-types */
#define	XNAM_SEM		0x01
#define	XNAM_SD			0x02

/*
 * Convert between vnode and xnamnode
 */
#define	VTOXNAM(vp)	((struct xnamnode *)((vp)->v_data))
#define	XNAMTOV(xp)	(&(xp)->x_vnode)

extern struct proc *curproc;	/* XXX vm debugging */
#if defined(mips) && !defined(u3b2v4) && !defined(lint)
#pragma large_data curproc
#endif
/*
 * Lock and unlock xnamnodes.
 */
#define XNAMNLOCK(xp) { \
	while ((xp)->x_flag & XNAMLOCKED) { \
		(xp)->x_flag |= XNAMWANT; \
		(void) sleep((caddr_t)(xp), PINOD); \
	} \
	(xp)->x_flag |= XNAMLOCKED; \
	if (((xp)->x_vnode.v_flag & VISSWAP) != 0) { \
		int smplvl;	\
		curproc->p_swlocks++; \
		SPLHI(smplvl);	\
		PSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
		curproc->p_flag |= SSWLOCKS; \
		VSPIN(&curproc->p_mplock, STAT_P_FLAG);			\
		SPLX(smplvl);	\
	} \
	(xp)->x_powns = curproc; \
}

#define XNAMNUNLOCK(xp) { \
	(xp)->x_flag &= ~XNAMLOCKED; \
	if (((xp)->x_vnode.v_flag & VISSWAP) != 0) \
		if (--curproc->p_swlocks == 0) { \
			int smplvl;	\
			SPLHI(smplvl);	\
			PSPIN(&curproc->p_mplock, STAT_P_FLAG);	\
			curproc->p_flag &= ~SSWLOCKS; \
			VSPIN(&curproc->p_mplock, STAT_P_FLAG);			\
			SPLX(smplvl);	\
		} \
	if ((xp)->x_flag & XNAMWANT) { \
		(xp)->x_flag &= ~XNAMWANT; \
		(void) wakeprocs((caddr_t)(xp), PRMPT); \
	} \
	(xp)->x_powns = NULL; \
}

/*
 * Construct an xnam vnode for a given device that shadows a particular
 * "real" vnode.
 */
extern struct vnode *xnamvp();


/*
 * Xnamnode lookup stuff.
 * These routines maintain a table of xnamnodes.
 */

#define	XNAMTABLESIZE	2
extern struct xnamnode *xnamtable[];

extern struct vnodeops xnam_vnodeops;

#if defined(_SMP) && defined(_FSCOMPAT)
/*
 * Declare global pointer to a pointer to the xnamfs compatibility lock.
 * It will be initialized in xnam_init().  We use a pointer to a pointer
 * rather than a pointer to the lock itself because when xnam_init() is
 * called, we don't yet have the lock allocated by vfs_init().  So we
 * "remember" where it WILL be when it gets set up and derefernce the
 * pointer later.
 */

extern	sema_t	**xnam_sema;	/* pointer to "our" semaphore lock */

/* Note that this is taken directly from genfs:mp_vnode.c */

/*
 * The following two macros manage any necessary locking/unlocking of
 * file system compatibility semaphores upon entry to and exit from an
 * xnamfs routine.  They MUST be placed inline within subroutines and
 * as a prerequisite, the following declaration must be made in each
 * such routine to hold any driver compatibility lock:
 *
 *	struct mp_sv_lcks svlcks;
 */

/*
 * This macro releases any previously held file system compatibility
 * semaphore, unless it is the same semaphore as the one to be acquired.
 * It then acquires the appropriate semaphore if it wasn't already held.
 */

#define PRE_XNAM_SERVICE(semap, relsemap) \
	ddmp_save_lcks(&svlcks, 0);	/* release driver compat. lock if necessary */ \
	/* if a compat. sema. for a different FSType is already held */ \
	if (u.u_fssema && u.u_fssema != *xnam_sema) { \
		semap = u.u_fssema; \
		VSEMA(semap, &mpsbuf[MPS_AUTO_FSCOMPAT]);	/* release previous compat. sema. */ \
		u.u_fssema = NULL; \
	} \
	/* if this file's FSType is in compat. mode and has not yet been acquired */ \
	if (u.u_fssema != *xnam_sema) { \
		/* acquire new compat. sema. */ \
		PSEMA(*xnam_sema, PZERO, &mpsbuf[MPS_AUTO_FSCOMPAT]); \
		u.u_fssema = *xnam_sema; \
		relsemap = *xnam_sema; \
	}

/*
 * This macro releases the appropriate file system compatibility
 * semaphore, if it was acquired above.  It then re-acquires any
 * file system compatibility semaphore which was released above.
 */

#define POST_XNAM_SERVICE(semap, relsemap) \
	if (relsemap) {	/* if a new compat. sema. was acquired */ \
		VSEMA(relsemap, &mpsbuf[MPS_AUTO_FSCOMPAT]); /* release it */ \
		u.u_fssema = NULL; \
	} \
	if (semap) {	/* if a previous compat. sema. was released */ \
		PSEMA(semap, PZERO, &mpsbuf[MPS_AUTO_FSCOMPAT]);	/* re-acquire it */ \
		u.u_fssema = semap; \
	} \
	ddmp_restore_lcks(&svlcks);	/* re-acquire driver compat. lock if necessary */

#else	/* !defined(_SMP) || !defined(_FSCOMPAT) */

#define PRE_XNAM_SERVICE(y, z)
#define POST_XNAM_SERVICE(y, z)

#endif	/* defined(_SMP) && defined(_FSCOMPAT) */

#endif	/* _FS_XNAMNODE_H */
