#ident "@(#)$Header: vfs.h,v 1.5 92/10/09 10:37:51 svr4 Exp $ SNI"
/*
 * "@(#)vfs.h	Revision: 3.1	92/05/28 19:34:53"
 */

/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990, 1991 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _SYS_VFS_H
#define _SYS_VFS_H

/*#ident	"@(#)ATTDSG gencpu/gensys/uts/sys/vfs.h	1.1.2.2"*/

#include <sys/conf.h>

/*
 * Data associated with mounted file systems.
 */

/*
 * File system identifier. Should be unique (at least per machine).
 */
typedef struct {
	long val[2];			/* file system id type */
} fsid_t;

/*
 * File identifier.  Should be unique per filesystem on a single
 * machine.  This is typically called by a stateless file server
 * in order to generate "file handles".
 */
#define	MAXFIDSZ	16
#define	freefid(fidp) \
  kmem_free((caddr_t)(fidp), sizeof (struct fid) - MAXFIDSZ + (fidp)->fid_len)

typedef struct fid {
	u_short		fid_len;		/* length of data in bytes */
	char		fid_data[MAXFIDSZ];	/* data (variable length) */
} fid_t;

/*
 * Structure per mounted file system.  Each mounted file system has
 * an array of operations and an instance record.  The file systems
 * are kept on a singly linked list headed by "rootvfs" and terminated
 * by NULL.
 *
 * The vfs_count field is used to reference count a vfs structure.  The
 * vfs_count and the VFS_MLOCK flag are similar to a reader/writer protocol.
 * A non-zero reference count prevents the VFS_MLOCK from succeeding and hence
 * an unmount of the filesystem from taking place.  Conversely, the VFS_MLOCK
 * prevents a process from using the vfs structure (to cross a mount point
 * for e.g.).
 */
typedef struct vfs {
	struct vfs	*vfs_next;		/* next VFS in VFS list */
	struct vfsops	*vfs_op;		/* operations on VFS */
	struct vnode	*vfs_vnodecovered;	/* vnode mounted on */
	u_long		vfs_flag;		/* flags */
	u_long		vfs_bsize;		/* native block size */
	int		vfs_fstype;		/* file system type index */
	fsid_t		vfs_fsid;		/* file system id */
	caddr_t		vfs_data;		/* private data */
	dev_t		vfs_dev;		/* device of mounted VFS */
	u_long		vfs_bcount;		/* I/O count (accounting) */
	u_short		vfs_nsubmounts;		/* immediate sub-mount count */
	u_short		vfs_count;		/* reference count */
 	sema_t          *vfs_sema;              /* filesystem compat. sema */
	short		vfs_owner;		/* process owning VFS_MLOCK */
#ifdef PYRSEC
	u_long		vfs_secflags;		/* security flags */
	caddr_t		vfs_devpath;		/* path for vfs_dev */
	u_long		vfs_filler[2];		/* future expansion */
#else
	caddr_t		vfs_devpath;		/* path for vfs_dev */
	u_long		vfs_filler[3];		/* future expansion */
#endif
} vfs_t;

/*
 * VFS flags.
 */
#define VFS_RDONLY	0x01		/* read-only vfs */
#define VFS_MLOCK	0x02		/* lock vfs so that subtree is stable */
#define VFS_MWAIT	0x04		/* someone is waiting for lock */
#define VFS_NOSUID	0x08		/* setuid disallowed */
#define VFS_REMOUNT	0x10		/* modify mount options only */
#define VFS_NOTRUNC	0x20		/* does not truncate long file names */
#define VFS_UNLINKABLE	0x40		/* unlink(2) can be applied to root */
#define VFS_BADBLOCK    0x80            /* disk based fs has bad block */
#define VFS_STHREAD	0x80000000      /* single-threaded filesystem */

/*
 * Calls for modifying the VFS flags in an SMP safe manner.
 */
#ifdef _SMP
#define VFS_SET(vfsp, flags)	vfs_setflags((vfsp), (flags))
#define VFS_CLEAR(vfsp, flags)	vfs_clearflags((vfsp), (flags))
#else /* _SMP */
#define VFS_SET(vfsp, flags)	(vfsp)->vfs_flag |= (flags)
#define VFS_CLEAR(vfsp, flags)	(vfsp)->vfs_flag &= ~(flags)
#endif /* _SMP */

/*
 * Argument structure for mount(2).
 */
struct mounta {
	char	*spec;
	char	*dir;
	int	flags;
	char	*fstype;
	char	*dataptr;
	int	datalen;
};

/*
 * Operations supported on virtual file system.
 */
typedef struct vfsops {
	int	(*vfs_mount)();		/* mount file system */
	int	(*vfs_unmount)();	/* unmount file system */
	int	(*vfs_root)();		/* get root vnode */
	int	(*vfs_statvfs)();	/* get file system statistics */
	int	(*vfs_sync)();		/* flush fs buffers */
	int	(*vfs_vget)();		/* get vnode from fid */
	int	(*vfs_mountroot)();	/* mount the root filesystem */
	int	(*vfs_swapvp)();	/* return vnode for swap */
#ifndef __pyrsoft
	int	(*vfs_filler[8])();	/* for future expansion */
#else
	int	(*vfs_fsinfo)();	/* return vnode for swap */
	int	(*vfs_filler[7])();	/* for future expansion */
#endif
} vfsops_t;

#ifdef __i386__
#define	_FSCOMPAT
#endif

#if defined(_SMP) && defined(_FSCOMPAT)

#define VFS_COMPAT(vfsp) (((vfsp)->vfs_flag & VFS_STHREAD) || \
			  (DDMP_STHREAD(u.u_drsp)) || \
			  (u.u_fssema != NULL))

#define VFS_MOUNT(vfsp, mvp, uap, cr)	(VFS_COMPAT(vfsp) ? \
					   mp_vfs_mount(vfsp, mvp, uap, cr) : \
					   (*(vfsp)->vfs_op->vfs_mount)(vfsp, mvp, uap, cr))
#define VFS_UNMOUNT(vfsp, cr)		(VFS_COMPAT(vfsp) ? \
					   mp_vfs_unmount(vfsp, cr) : \
					   (*(vfsp)->vfs_op->vfs_unmount)(vfsp, cr))
#define VFS_ROOT(vfsp, vpp)		(VFS_COMPAT(vfsp) ? \
					   mp_vfs_root(vfsp, vpp) : \
					   (*(vfsp)->vfs_op->vfs_root)(vfsp, vpp))
#define VFS_STATVFS(vfsp, sp)		(VFS_COMPAT(vfsp) ? \
					   mp_vfs_statvfs(vfsp, sp) : \
					   (*(vfsp)->vfs_op->vfs_statvfs)(vfsp, sp))
#define VFS_SYNC(vfsp, flag, cr)	(VFS_COMPAT(vfsp) ? \
					   mp_vfs_sync(vfsp, flag, cr) : \
					   (*(vfsp)->vfs_op->vfs_sync)(vfsp, flag, cr))
#define VFS_VGET(vfsp, vpp, fidp)	(VFS_COMPAT(vfsp) ? \
					   mp_vfs_vget(vfsp, vpp, fidp) : \
					   (*(vfsp)->vfs_op->vfs_vget)(vfsp, vpp, fidp))
#define VFS_MOUNTROOT(vfsp, init)	(VFS_COMPAT(vfsp) ? \
					   mp_vfs_mountroot(vfsp, init) : \
		 			   (*(vfsp)->vfs_op->vfs_mountroot)(vfsp, init))
#define VFS_SWAPVP(vfsp, vpp, nm)	(VFS_COMPAT(vfsp) ? \
					   mp_vfs_swapvp(vfsp, vpp, nm) : \
					   (*(vfsp)->vfs_op->vfs_swapvp)(vfsp, vpp, nm))
#ifdef __pyrsoft
#define VFS_FSINFO(vfsp, buf)	(VFS_COMPAT(vfsp) ? \
					   mp_vfs_fsinfo(vfsp, buf) : \
					   (*(vfsp)->vfs_op->vfs_fsinfo)(vfsp, buf))
#endif

#else	/* defined(_SMP) && defined(_FSCOMPAT) */

#define VFS_MOUNT(vfsp, mvp, uap, cr) \
	(*(vfsp)->vfs_op->vfs_mount)(vfsp, mvp, uap, cr)
#define VFS_UNMOUNT(vfsp, cr)	(*(vfsp)->vfs_op->vfs_unmount)(vfsp, cr)
#define VFS_ROOT(vfsp, vpp)	(*(vfsp)->vfs_op->vfs_root)(vfsp, vpp)
#define	VFS_STATVFS(vfsp, sp)	(*(vfsp)->vfs_op->vfs_statvfs)(vfsp, sp)
#define VFS_SYNC(vfsp, flag, cr) \
		(*(vfsp)->vfs_op->vfs_sync)(vfsp, flag, cr)
#define VFS_VGET(vfsp, vpp, fidp) \
		(*(vfsp)->vfs_op->vfs_vget)(vfsp, vpp, fidp)
#define VFS_MOUNTROOT(vfsp, init) \
		 (*(vfsp)->vfs_op->vfs_mountroot)(vfsp, init)
#define VFS_SWAPVP(vfsp, vpp, nm) \
		(*(vfsp)->vfs_op->vfs_swapvp)(vfsp, vpp, nm)

#ifdef __pyrsoft
#define VFS_FSINFO(vfsp, buf)	(*(vfsp)->vfs_op->vfs_fsinfo)(vfsp, buf)
#endif

#endif	/* defined(_SMP) && defined(_FSCOMPAT) */

/*
 * Filesystem type switch table.
 */
typedef struct vfssw {
	char		*vsw_name;	/* type name string */
	int		(*vsw_init)();	/* init routine */
	struct vfsops	*vsw_vfsops;	/* filesystem operations vector */
	long		vsw_flag;	/* flags & fs type (see below) */
	sema_t		*vsw_sema;	/* filesystem compat. semaphore */
} vfssw_t;

/*
 * Filesystem type flags.
 */
#define F_STHREAD	0x01		/* single-threaded filesystem type */

#define	F_TC_MASK	0x00f000	/* category (disk based or kernel based) */
#define	F_TS_MASK	0x0f0000	/* sub-category (local or remote) */
#define	F_T_MASK	0xfff000
#define	F_T_SHIFT	12
#define	F_TS_SHIFT	16

#define	F_TC_DISK	0x001000	/* disk-based FS */
#define	F_TC_KERNEL	0x002000	/* kernel-based FS */

#define	F_TS_LOCAL	0x010000	/* a local FS (e.g. ufs) */
#define	F_TS_REMOTE	0x020000	/* a remote (network) FS (e.g. nfs) */
#define	F_TS_INTERNAL	0x040000	/* internal FS (no user interface - e.g. fifofs,specfs) */
#define	F_TS_APPLIC	0x080000	/* application FS (user interface - e.g. proc,fdfs) */


#define	VFSSW_FLAGS2TYPE(flags)		(((flags) & F_T_MASK) >> F_T_SHIFT)

/*
 * Public operations.
 *
 * ##_SMP The semantics of some of these operations are different for SMP to
 * account for a race between a process attempting to use a vfs structure and
 * an unmount of the underlying filesystem.  The changes are as follows.
 * 	vfs_devsearch() and getvfs() return a vfs that has been reference 
 *		counted via a vfs_hold().  Hence, the caller must call 
 * 		vfs_rele() after using the vfs structure.
 *	dounmount() expects the caller to do a vfs_hold() before the call to 
 *		ensure that it is not unmounted by another process.  The
 *		corresponding vfs_rele() is done by this operation.
 */
extern void	vfs_mountroot();	/* mount the root */
extern void	vfs_add();		/* add a new vfs to mounted vfs list */
extern void	vfs_remove();		/* remove a vfs from mounted vfs list */
extern int	vfs_lock();		/* lock a vfs */
extern void	vfs_unlock();		/* unlock a vfs */
extern int	vfs_hold();		/* hold a vfs */
extern void	vfs_rele();		/* release a vfs */
extern vfs_t 	*getvfs();		/* return vfs given fsid */
extern vfs_t 	*vfs_devsearch();	/* find vfs given device */
extern vfssw_t 	*vfs_getvfssw();	/* find vfssw ptr given fstype name */
extern u_long	vf_to_stf();		/* map VFS flags to statfs flags */
extern int	dounmount();		/* unmount a vfs */

#define VFS_INIT(vfsp, op, data)	{ \
	(vfsp)->vfs_next = (struct vfs *)0; \
	(vfsp)->vfs_op = (op); \
	(vfsp)->vfs_flag = 0; \
	(vfsp)->vfs_data = (data); \
	(vfsp)->vfs_nsubmounts = 0; \
	(vfsp)->vfs_count = 0; \
}

/*
 * Globals.
 */
extern struct vfs *rootvfs;		/* ptr to root vfs structure */
extern struct vfssw vfssw[];		/* table of filesystem types */
extern char rootfstype[];		/* name of root fstype */
extern int nfstype;			/* # of elements in vfssw array */

/*
 * Reasons for calling the vfs_mountroot() operation.
 *
 * This is slightly changed from standard SVR4, because ufs has common code
 * to mountroot and mountfs. So, to make things clearer, we added FS_*
 */

/* BOOT_INIT and BOOT_UNMOUNT are added to mount/unmount the boot device */
/* to implement mirror root */
enum whymountroot { ROOT_INIT, ROOT_REMOUNT, ROOT_UNMOUNT, FS_INIT,
		    FS_REMOUNT, BOOT_INIT, BOOT_UNMOUNT };
typedef enum whymountroot whymountroot_t;

/*
 * VFS_SYNC flags.
 */
#define SYNC_ATTR	0x01		/* sync attributes only */
#define SYNC_CLOSE	0x02		/* close open file */
#define SYNC_PANIC	0x04		/* sync following a panic */

#ifdef __pyrsoft
/*
 * The following is for the sysmips(GETFSINFO) command.
 */

/*
 * Information about a specific file system.
 */
struct userfsinfo_spec {
	u_long	fsi_flag;		/* (see below) */
	u_long	fsi_type;		/* (see below) */
	int	fsi_whichfs;		/* file system type index */
	u_long	fsi_size;		/* size of FS (in DEV_BSIZE units) */
	dev_t	fsi_dev;		/* device of mounted VFS */
	char	*fsi_devpath;		/* path of mounted VFS */
	char	*fsi_mntpt;		/* mount point */
	int	filler[26];
};

/*
 * sysmips() command structure.
 */
struct userfsinfo {
	int	fsi_cmd;		/* - includes versioning (see below) */
	int	fsi_nmounts;		/* # of mounted filesystems */
	int	fsi_index;		/* info on specific fs (0..fsm_nmounts-1) */
	int	fsi_gen;		/* changes for each mount/umount */
					/* (used to get consistent image of mount table) */
	struct userfsinfo_spec	fsi_spec;
};

/*
 * userfsinfo commands.
 */
#define	GETFSC_NMOUNTS		1		/* fill in nmounts and gen only */
#define	GETFSC_MNTLIST		2		/* get info on mounted fs specified by index */
#define	GETFSC_FSTYPETYPE	3		/* get info on FSType specified by index */
#define	GETFSC_FSISMOUNTED	4		/* see if the given dev is mounted */
#define	GETFSC_GETMNTENT	5		/* get info on mounted fs specified by dev */

#define	GETFS_VMASK		0xf0000000
#define	GETFS_VSHIFT		28
#define	GETFS_VERSION		0

/*
 * userfsinfo flags (GETFS_MNTLIST only).
 * (these are user versions of (relevant) vfs flags)
 */
#define	GETFSF_RDONLY		0x00000001		/* read-only */
#define	GETFSF_NOSUID		0x00000002		/* setuid disallowed */
#define	GETFSF_NOTRUNC		0x00000004		/* does not truncate long file names */
#define	GETFSF_UNLINKABLE	0x00000008		/* unlink(2) can be applied to root */
#define	GETFSF_BADBLOCK		0x00000010		/* fs has bad block */

/*
 * userfsinfo type (GETFS_MNTLIST, GETFS_TYPETYPE only).
 * (see vfssw flags above)
 */
#define	GETFST_CMASK	0x0f	/* category (disk based or kernel based) */
#define	GETFST_SMASK	0xf0	/* sub-category (local or remote) */
#define	GETFST_MASK	0xff

#define	GETFST_DISK	0x01	/* disk-based FS */
#define	GETFST_KERNEL	0x02	/* kernel-based FS */

#define	GETFST_LOCAL	0x10	/* a local FS (e.g. ufs) */
#define	GETFST_REMOTE	0x20	/* a remote (network) FS (e.g. nfs) */
#define	GETFST_INTERNAL	0x40	/* internal FS (no user interface - e.g. fifofs,specfs) */
#define	GETFST_APPLIC	0x80	/* application FS (user interface - e.g. proc,fdfs) */


#endif

#endif	/* _SYS_VFS_H */
